package nl.kallestruik.dtweaks.commands

import co.aikar.commands.BaseCommand
import co.aikar.commands.annotation.CommandAlias
import co.aikar.commands.annotation.Conditions
import co.aikar.commands.annotation.Subcommand
import net.kyori.adventure.text.Component
import net.kyori.adventure.text.format.NamedTextColor
import nl.kallestruik.dtweaks.managers.NoSpawnZoneManager
import org.bukkit.entity.Player


@CommandAlias("spawnzones")
@Conditions("tweakEnabled:tweak=NoSpawnZones")
class CommandSpawnzones(
    private val noSpawnZoneManager: NoSpawnZoneManager
): BaseCommand() {

    @Subcommand("claim")
    inner class CommandClaim: BaseCommand() {

        @Subcommand("one")
        fun onClaimOne(sender: Player) {
            noSpawnZoneManager.claimChunk(sender.location.chunk.x, sender.location.chunk.z, sender.world.key)
            noSpawnZoneManager.saveToFile()
        }

        @Subcommand("square")
        fun onClaimSquare(sender: Player, radius: Int) {
            val centerX = sender.location.chunk.x
            val centerZ = sender.location.chunk.z


            for (x in centerX-radius..centerX+radius)
                for (z in centerZ-radius..centerZ+radius)
                    noSpawnZoneManager.claimChunk(x, z, sender.world.key)

            noSpawnZoneManager.saveToFile()
        }
    }

    @Subcommand("unclaim")
    inner class CommandUnclaim: BaseCommand() {

        @Subcommand("one")
        fun onClaimOne(sender: Player) {
            noSpawnZoneManager.unclaimChunk(sender.location.chunk.x, sender.location.chunk.z, sender.world.key)
            noSpawnZoneManager.saveToFile()
        }

        @Subcommand("square")
        fun onClaimSquare(sender: Player, radius: Int) {
            val centerX = sender.location.chunk.x
            val centerZ = sender.location.chunk.z

            for (x in centerX-radius..centerX+radius)
                for (z in centerZ-radius..centerZ+radius)
                    noSpawnZoneManager.unclaimChunk(x, z, sender.world.key)

            noSpawnZoneManager.saveToFile()
        }
    }

    @Subcommand("map")
    fun onMap(sender: Player) {
        // Print the header of the faction map.
        sender.sendMessage(Component
            .empty()
            .color(NamedTextColor.GRAY)
            .append(Component.text("---------------["))
            .append(Component
                .text(" No Spawn Zones ")
                .color(NamedTextColor.AQUA))
            .append(Component.text("]---------------"))
        )

        val centerX = sender.location.chunk.x
        val centerZ = sender.location.chunk.z

        // Create and send the claim map.
        for (x in -5..5) {
            var row = Component.empty()
            for (z in -6..6) {
                row = if (x == 0 && z == 0) {
                    row.append(Component.text(" ■").color(NamedTextColor.YELLOW))
                } else if (noSpawnZoneManager.isChunkClaimed(centerX + x, centerZ + z, sender.world.key)) {
                    row.append(Component.text(" ■").color(NamedTextColor.AQUA))
                } else {
                    row.append(Component.text(" ■").color(NamedTextColor.GRAY))
                }
            }
            sender.sendMessage(row)
        }

        // Print the footer of the faction map.
        sender.sendMessage(Component
            .text("---------------------------------------")
            .color(NamedTextColor.GRAY)
        )
    }
}