package nl.kallestruik.dtweaks.tweaks.dispsensertweaks

import nl.kallestruik.dtweaks.tweaks.ITweak
import org.bukkit.Material
import org.bukkit.Particle
import org.bukkit.Tag
import org.bukkit.block.BlockFace
import org.bukkit.block.data.type.Dispenser
import org.bukkit.event.EventHandler
import org.bukkit.event.HandlerList
import org.bukkit.event.Listener
import org.bukkit.event.block.BlockDispenseEvent
import org.bukkit.plugin.java.JavaPlugin

class DispensersCanPlantSaplings(
    private val plugin: JavaPlugin
): ITweak, Listener {
    private val treePlantable = setOf(Material.DIRT, Material.COARSE_DIRT, Material.GRASS_BLOCK, Material.PODZOL, Material.MYCELIUM)

    override fun getIdentifier(): String = "DispensersCanPlantSaplings"
    override fun getCategories(): List<String> = listOf("dispenser", "survival")

    override fun onEnable() {
        plugin.server.pluginManager.registerEvents(this, plugin)
    }

    override fun onDisable() {
        HandlerList.unregisterAll(this)
    }

    @EventHandler
    fun onDispenserDispense(event: BlockDispenseEvent) {
        if (event.block.type != Material.DISPENSER) return
        if (Tag.SAPLINGS.isTagged(event.item.type)) {
            val dispenser = event.block.blockData as Dispenser
            val facing = dispenser.facing
            val targetBlock = if (facing == BlockFace.UP) {
                event.block.world.getBlockAt(event.block.location.add(0.0, 2.0, 0.0))
            } else {
                event.block.world.getBlockAt(
                    event.block.location.add(
                        facing.modX.toDouble(),
                        facing.modY.toDouble(),
                        facing.modZ.toDouble()
                    )
                )
            }
            if (targetBlock.type == Material.AIR
                && treePlantable.contains(
                    event.block.world.getBlockAt(
                        targetBlock.location.add(
                            0.0,
                            -1.0,
                            0.0
                        )
                    ).type
                )
            ) {
                targetBlock.type = event.item.type
                event.isCancelled = true
                val dispenserInventory = (event.block.state as org.bukkit.block.Dispenser).inventory
                var slot = 0
                if (dispenserInventory.contents == null)
                    return

                for (item in dispenserInventory.contents!!) {
                    if (item != null && item.type == event.item.type) break
                    slot++
                }
                val newItemStack = dispenserInventory.getItem(slot)
                newItemStack!!.amount = newItemStack.amount - 1
                dispenserInventory.setItem(slot, newItemStack)
                event.block.world.spawnParticle(Particle.VILLAGER_HAPPY, targetBlock.location, 5)
            }
        }
    }
}